"""Exec module for managing Subscriptions."""
from collections import OrderedDict
from typing import Any
from typing import Dict

__func_alias__ = {"list_": "list"}


async def get(hub, ctx, resource_id: str, raw: bool = False) -> Dict[str, Any]:
    """Get subscription resource from resource_id.

    Args:
        resource_id(str):
            The resource_id of subscription.
        raw(bool, Optional):
            Returns raw response if True. Defaults to False

    Returns:
        Dict[str, Any]

    Examples:
        Calling this exec module function from the cli with resource_id:

        .. code-block:: bash

            idem exec azure.subscription.subscriptions.get resource_id="value"

        Using in a state:

        .. code-block:: yaml

            my_unmanaged_resource:
              exec.run:
                - path:  azure.subscription.subscriptions.get
                - kwargs:
                    resource_id: "/providers/Microsoft.Subscription/aliases/{alias}"

    """
    result = dict(comment=[], result=True, ret=None)
    uri_parameters = OrderedDict({"aliases": "alias"})

    response_get = await hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2020-09-01",
        success_codes=[200],
    )
    if not response_get["result"]:
        if not response_get["ret"] != 404:
            result["result"] = False
        result["comment"] = response_get["comment"]
        return result

    elif response_get["result"] and response_get["ret"]:
        subscription_id = response_get["ret"]["properties"]["subscriptionId"]
        response_get_detailed_subscription = await hub.exec.request.json.get(
            ctx,
            url=f"{ctx.acct.endpoint_url}/subscriptions/{subscription_id}?api-version=2020-01-01",
            success_codes=[200],
        )
        if not response_get_detailed_subscription["result"]:
            result["result"] = False
            result["comment"] = response_get_detailed_subscription["comment"]
            return result
        else:
            if raw:
                result["ret"] = response_get_detailed_subscription["ret"]
            else:
                uri_parameter_values = hub.tool.azure.uri.get_parameter_value_in_dict(
                    resource_id, uri_parameters
                )
                result[
                    "ret"
                ] = hub.tool.azure.subscription.subscriptions.convert_raw_subscription_to_present(
                    resource=response_get_detailed_subscription["ret"],
                    idem_resource_name=resource_id,
                    resource_id=resource_id,
                    subscription_id=subscription_id,
                    display_name=None,
                    **uri_parameter_values,
                )

    return result


async def list_(hub, ctx) -> Dict:
    """List of subscriptions

    Returns:
        Dict[str, Any]

    Examples:
        Calling this exec module function from the cli with resource_id:

        .. code-block:: bash

            idem exec azure.subscription.subscriptions.list

        Using in a state:

        .. code-block:: yaml

            my_unmanaged_resource:
              exec.run:
                - path: azure.subscription.subscriptions.list


    """
    result = dict(comment=[], result=True, ret=[])
    uri_parameters = OrderedDict({"subscriptions": "subscription_id"})
    async for page_result in hub.tool.azure.request.paginate(
        ctx,
        url=f"{ctx['acct']['endpoint_url']}/subscriptions?api-version=2020-01-01",
        success_codes=[200],
    ):
        resource_list = page_result.get("value", None)
        if resource_list:
            for resource in resource_list:
                resource_id = resource["id"]
                uri_parameter_values = hub.tool.azure.uri.get_parameter_value_in_dict(
                    resource["id"], uri_parameters
                )
                result["ret"].append(
                    hub.tool.azure.subscription.subscriptions.convert_raw_subscription_to_present(
                        resource=resource,
                        idem_resource_name=resource_id,
                        resource_id=resource_id,
                        alias=None,
                        display_name=None,
                        **uri_parameter_values,
                    )
                )

    return result



"""Exec module for managing Firewall Policy."""
from collections import OrderedDict
from typing import Any
from typing import Dict

__func_alias__ = {"list_": "list"}


async def get(
    hub,
    ctx,
    resource_id: str,
    raw: bool = False,
) -> Dict[str, Any]:
    """Get firewall policy resource from resource_id.

    Args:
        resource_id(str):
            The resource_id of firewall policy
        raw(bool):
            Returns raw response if True. Defaults to False

    Returns:
        Dict[str, Any]

    Examples:
        Calling this exec module function from the cli with resource_id:

        .. code-block:: bash

            idem exec azure.network.firewall_policies.get resource_id="value"

        Using in a state:

        .. code-block:: yaml

            my_unmanaged_resource:
              exec.run:
                - path:  azure.network.firewall_policies.get
                - kwargs:
                    resource_id: "/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/firewallPolicies/{firewall_policy_name}"

    """
    result = dict(comment=[], result=True, ret=None)
    uri_parameters = OrderedDict(
        {
            "resourceGroups": "resource_group_name",
            "subscriptions": "subscription_id",
            "firewallPolicies": "firewall_policy_name",
        }
    )

    response_get = await hub.exec.request.json.get(
        ctx,
        url=f"{hub.exec.azure.URL}{resource_id}?api-version=2022-07-01",
        success_codes=[200],
    )
    if not response_get["result"]:
        if response_get["status"] != 404:
            result["result"] = False
        result["comment"] = response_get["comment"]
        return result

    elif response_get["result"] and response_get["ret"]:
        if raw:
            result["ret"] = response_get["ret"]

        else:
            uri_parameter_values = hub.tool.azure.uri.get_parameter_value_in_dict(
                resource_id, uri_parameters
            )
            result[
                "ret"
            ] = hub.tool.azure.network.firewall_policies.convert_raw_firewall_policies_to_present(
                resource=response_get["ret"],
                idem_resource_name=resource_id,
                resource_id=resource_id,
                **uri_parameter_values,
            )

    return result


async def list_(hub, ctx) -> Dict:
    """List of firewall policies

    Returns:
        Dict[str, Any]

    Examples:
        Calling this exec module function from the cli with resource_id:

        .. code-block:: bash

            idem exec azure.network.firewall_policies.list

        Using in a state:

        .. code-block:: yaml

            my_unmanaged_resource:
              exec.run:
                - path: azure.network.firewall_policies


    """
    result = dict(comment=[], result=True, ret=[])
    subscription_id = ctx.acct.subscription_id
    uri_parameters = OrderedDict(
        {
            "resourceGroups": "resource_group_name",
            "subscriptions": "subscription_id",
            "firewallPolicies": "firewall_policy_name",
        }
    )
    async for page_result in hub.tool.azure.request.paginate(
        ctx,
        url=f"{ctx.acct.endpoint_url}/subscriptions/{subscription_id}/providers/Microsoft.Network/firewallPolicies?api-version=2022-07-01",
        success_codes=[200],
    ):
        firewall_policy_list = page_result.get("value")
        if firewall_policy_list:
            for resource in firewall_policy_list:
                resource_id = resource["id"]
                uri_parameter_values = hub.tool.azure.uri.get_parameter_value_in_dict(
                    resource["id"], uri_parameters
                )
                result["ret"].append(
                    hub.tool.azure.network.firewall_policies.convert_raw_firewall_policies_to_present(
                        resource=resource,
                        idem_resource_name=resource_id,
                        resource_id=resource_id,
                        **uri_parameter_values,
                    )
                )
    return result



"""State module for managing Compute Log Analytics Workspace."""
import copy
from collections import OrderedDict
from typing import Any
from typing import Dict

__contracts__ = ["resource"]


async def present(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    subscription_id: str,
    workspace_name: str,
    location: str,
    eTag: str = None,
    default_data_collection_rule_resourceId: str = None,
    features: Dict = None,
    force_cmk_for_query: bool = False,
    public_network_access_for_ingestion: str = None,
    public_network_access_for_query: str = None,
    retention_in_days: int = None,
    sku: Dict = None,
    workspace_capping: Dict = None,
    tags: Dict = None,
    resource_id: str = None,
) -> Dict:
    r"""Create or update Log Analytics Workspace.

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group. The name is case insensitive.
        workspace_name(str): The name of the workspace. Regex pattern: ^[A-Za-z0-9][A-Za-z0-9-]+[A-Za-z0-9]$
        eTag(str): The ETag of the workspace.
        location(str): Resource location. This field can not be updated.
        resource_id(str, Optional): Workspace resource id on Azure
        default_data_collection_rule_resourceId(str): The resource ID of the default Data Collection Rule to use for this workspace.
        features(dict, Optional): Workspace features.
        force_cmk_for_query(dict, Optional): Indicates whether customer managed storage is mandatory for query management.
        public_network_access_for_ingestion(str, Optional): The network access type for accessing Log Analytics ingestion.
        public_network_access_for_query(str, Optional): The network access type for accessing Log Analytics query.
        retention_in_days(int, Optional): The workspace data retention in days. Allowed values are per pricing plan. See pricing tiers documentation for details.
        sku(dict, Optional): The SKU of the workspace.
        workspace_capping(dict): The daily volume cap for ingestion.
        tags(dict, Optional): Resource tags.
        subscription_id(str): Subscription Unique id.

    Returns:
        dict

    Examples:
        .. code-block:: sls

            workspaces/sb-instance:
             azure.compute.log_analytics_workspace.present:
              - name: /subscriptions/98bf10bb-cf16-49eb-9ef3-7ccc82e5d45e/resourcegroups/sb-resource-group/providers/Microsoft.OperationalInsights/workspaces/sb-workspace
              - resource_id: /subscriptions/98bf10bb-cf16-49eb-9ef3-7ccc82e5d45e/resourcegroups/sb-resource-group/providers/Microsoft.OperationalInsights/workspaces/sb-workspace
              - resource_group_name: sb-resource-group
              - workspace_name: sb-workspace
              - location: westus
              - subscription_id: 98bf10bb-cf16-49eb-9ef3-7ccc82e5d45e
              - tags:
                  E2ETag: E2EValue
                  Environment: Next
                  env: next
              - sku:
                  lastSkuUpdate: '2022-12-13T05:51:07.1724412Z'
                  name: pergb2018
              - features:
                  enableLogAccessUsingOnlyResourcePermissions: true
                  legacy: 0
                  searchVersion: 1
              - workspace_capping:
                  dailyQuotaGb: -1.0
                  dataIngestionStatus: RespectQuota
                  quotaNextResetTime: '2022-12-13T08:00:00Z'
              - retention_in_days: 30
              - public_network_access_for_ingestion: Enabled
              - public_network_access_for_query: Enabled
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }

    if subscription_id is None:
        subscription_id = ctx.acct.subscription_id
    if resource_id is None:
        resource_id = f"/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.OperationalInsights/workspaces/{workspace_name}"
    response_get = await hub.exec.azure.compute.log_analytics_workspace.get(
        ctx, resource_id=resource_id, raw=True
    )
    if not response_get["result"]:
        hub.log.debug(
            f"Could not get azure.compute.log_analytics_workspace {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result

    if not response_get["ret"]:
        if ctx.get("test", False):
            # Return a proposed state by Idem state --test
            result["new_state"] = hub.tool.azure.test_state_utils.generate_test_state(
                enforced_state={},
                desired_state={
                    "name": name,
                    "resource_group_name": resource_group_name,
                    "subscription_id": subscription_id,
                    "workspace_name": workspace_name,
                    "location": location,
                    "eTag": eTag,
                    "default_data_collection_rule_resourceId": default_data_collection_rule_resourceId,
                    "features": features,
                    "force_cmk_for_query": force_cmk_for_query,
                    "public_network_access_for_ingestion": public_network_access_for_ingestion,
                    "public_network_access_for_query": public_network_access_for_query,
                    "retention_in_days": retention_in_days,
                    "sku": sku,
                    "workspace_capping": workspace_capping,
                    "tags": tags,
                    "resource_id": resource_id,
                },
            )
            result["comment"].append(
                f"Would create azure.compute.log_analytics_workspace '{name}'"
            )
            return result
        else:
            # PUT operation to create a resource
            payload = hub.tool.azure.compute.log_analytics_workspace.convert_present_log_analytics_workspace_to_raw(
                subscription_id=subscription_id,
                public_network_access_for_query=public_network_access_for_query,
                public_network_access_for_ingestion=public_network_access_for_ingestion,
                retention_in_days=retention_in_days,
                workspace_capping=workspace_capping,
                features=features,
                sku=sku,
                location=location,
                tags=tags,
            )
            response_put = await hub.exec.request.json.put(
                ctx,
                url=f"{hub.exec.azure.URL}{resource_id}?api-version=2021-12-01-preview",
                success_codes=[200, 201],
                json=payload,
            )

            if not response_put["result"]:
                hub.log.debug(
                    f"Could not create azure.compute.log_analytics {response_put['comment']} {response_put['ret']}"
                )
                result["comment"] = [response_put["comment"], response_put["ret"]]
                result["result"] = False
                return result

            result[
                "new_state"
            ] = hub.tool.azure.compute.log_analytics_workspace.convert_raw_log_analytics_workspace_to_present(
                resource=response_put["ret"],
                idem_resource_name=name,
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                resource_id=resource_id,
                subscription_id=subscription_id,
            )
            result["comment"].append(
                f"Created azure.compute.log_analytics_workspace '{name}'"
            )
            return result

    else:
        existing_resource = response_get["ret"]
        result[
            "old_state"
        ] = hub.tool.azure.compute.log_analytics_workspace.convert_raw_log_analytics_workspace_to_present(
            resource=existing_resource,
            idem_resource_name=name,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            resource_id=resource_id,
            subscription_id=subscription_id,
        )
        # Generate a new PUT operation payload with new values
        new_payload = hub.tool.azure.compute.log_analytics_workspace.update_log_analytics_workspace_payload(
            subscription_id,
            response_get["ret"],
            {
                "retention_in_days": retention_in_days,
                "public_network_access_for_ingestion": public_network_access_for_ingestion,
                "public_network_access_for_query": public_network_access_for_query,
                "tags": tags,
                "features": features,
                "sku": sku,
                "workspace_capping": workspace_capping,
            },
        )
        if ctx.get("test", False):
            if new_payload["ret"] is None:
                result["new_state"] = copy.deepcopy(result["old_state"])
                result["comment"].append(
                    f"azure.compute.log_analytics_workspace '{name}' has no property need to be updated."
                )
            else:
                result[
                    "new_state"
                ] = hub.tool.azure.compute.log_analytics_workspace.convert_raw_log_analytics_workspace_to_present(
                    resource=new_payload["ret"],
                    idem_resource_name=name,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    resource_id=resource_id,
                    subscription_id=subscription_id,
                )
                result["comment"].append(
                    f"Would update azure.compute.log_analytics_workspace '{name}'"
                )
            return result
        # PUT operation to update a resource
        if new_payload["ret"] is None:
            result["new_state"] = copy.deepcopy(result["old_state"])
            result["comment"].append(
                f"azure.compute.log_analytics_workspace '{name}' has no property need to be updated."
            )
            return result
        result["comment"] += new_payload["comment"]
        response_put = await hub.exec.request.json.put(
            ctx,
            url=f"{hub.exec.azure.URL}{resource_id}?api-version=2021-12-01-preview",
            success_codes=[200],
            json=new_payload["ret"],
        )

        if not response_put["result"]:
            hub.log.debug(
                f"Could not update azure.compute.log_analytics_workspace {response_put['comment']} {response_put['ret']}"
            )
            result["result"] = False
            result["comment"] = [response_put["comment"], response_put["ret"]]
            return result

        result[
            "new_state"
        ] = hub.tool.azure.compute.log_analytics_workspace.convert_raw_log_analytics_workspace_to_present(
            resource=response_put["ret"],
            idem_resource_name=name,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            resource_id=resource_id,
            subscription_id=subscription_id,
        )
        result["comment"].append(
            f"Updated azure.compute.log_analytics_workspace '{name}'"
        )
        return result


async def absent(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    workspace_name: str,
    subscription_id: str = None,
) -> Dict:
    r"""Delete a Log Analytics Workspace.

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group.
        workspace_name(str): The name of the workspace.
        subscription_id(str, Optional): Subscription Unique id.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_absent:
              azure.compute.log_analytics_workspace.absent:
                - name: my-workspace
                - resource_group_name: my-resource-group
                - workspace_name: my-workspace
                - subscription_id: my-subscription
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }
    if subscription_id is None:
        subscription_id = ctx.acct.subscription_id
    resource_id = (
        f"/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}"
        f"/providers/Microsoft.OperationalInsights/workspaces/{workspace_name}"
    )
    response_get = await hub.exec.azure.compute.log_analytics_workspace.get(
        ctx,
        resource_id=resource_id,
    )
    if not response_get["result"]:
        hub.log.debug(
            f"Could not get azure.compute.log_analytics_workspace '{name}' {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result

    if response_get["ret"]:
        result["old_state"] = response_get["ret"]
        result["old_state"]["name"] = name
        if ctx.get("test", False):
            result["comment"].append(
                f"Would delete azure.compute.log_analytics_workspace '{name}'"
            )
            return result

        response_delete = await hub.exec.request.raw.delete(
            ctx,
            url=f"{hub.exec.azure.URL}{resource_id}?api-version=2021-12-01-preview",
            success_codes=[200, 202, 204],
        )
        if not response_delete["result"]:
            hub.log.debug(
                f"Could not delete azure.compute.log_analytics_workspace {response_delete['comment']} {response_delete['ret']}"
            )
            result["result"] = False
            result["comment"] = [response_delete["comment"], response_delete["ret"]]
            return result

        result["comment"].append(
            f"Deleted azure.compute.log_analytics_workspace '{name}'"
        )
        return result
    else:
        # If Azure returns 'Not Found' error, it means the resource has been absent.
        result["comment"].append(
            f"azure.compute.log_analytics_workspace '{name}' already absent"
        )
        return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""Describe the resource in a way that can be recreated/managed with the corresponding "present" function.

    Lists all Log Analytics Workspace under the same subscription


    Returns:
        Dict[str, Any]

    Examples:
        .. code-block:: bash

            $ idem describe azure.compute.log_analytics_workspace
    """
    result = {}
    subscription_id = ctx.acct.subscription_id
    uri_parameters = OrderedDict(
        {
            "resourceGroups": "resource_group_name",
            "resourcegroups": "resource_group_name",
            "workspaces": "workspace_name",
        }
    )
    async for page_result in hub.tool.azure.request.paginate(
        ctx,
        url=f"{hub.exec.azure.URL}/subscriptions/{subscription_id}"
        f"/providers/Microsoft.OperationalInsights/workspaces?api-version=2021-12-01-preview",
        success_codes=[200],
    ):
        resource_list = page_result.get("value", None)
        if resource_list:
            for resource in resource_list:
                resource_id = resource["id"]
                uri_parameter_values = hub.tool.azure.uri.get_parameter_value_in_dict(
                    resource_id, uri_parameters
                )
                resource_translated = hub.tool.azure.compute.log_analytics_workspace.convert_raw_log_analytics_workspace_to_present(
                    resource=resource,
                    idem_resource_name=resource_id,
                    subscription_id=subscription_id,
                    resource_id=resource_id,
                    **uri_parameter_values,
                )
                result[resource_id] = {
                    f"azure.compute.log_analytics_workspace.present": [
                        {parameter_key: parameter_value}
                        for parameter_key, parameter_value in resource_translated.items()
                    ]
                }
    return result



"""State module for managing Compute Virtual Machine."""
import copy
from dataclasses import make_dataclass
from typing import Any
from typing import Dict
from typing import List

__contracts__ = ["resource"]
__reconcile_wait__ = {"static": {"wait_in_seconds": 20}}


async def present(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    virtual_machine_name: str,
    location: str,
    virtual_machine_size: str,
    network_interface_ids: List[str],
    os_profile: make_dataclass(
        "OsProfile",
        [
            ("computer_name", str),
            ("admin_username", str),
            ("admin_password", str),
        ],
    ),
    storage_image_reference: make_dataclass(
        "OsProfile",
        [
            ("image_sku", str),
            ("image_publisher", str),
            ("image_version", str),
            ("image_offer", str),
        ],
    ),
    storage_os_disk: make_dataclass(
        "StorageOsDisk",
        [
            ("disk_name", str),
            ("disk_size_in_GB", int),
            ("disk_caching", str),
            ("disk_create_option", str),
            ("disk_delete_option", str),
            ("disk_id", str),
            ("storage_account_type", str),
        ],
    ),
    storage_data_disks: List[
        make_dataclass(
            "StorageDataDisks",
            [
                ("disk_name", str),
                ("disk_size_in_GB", int),
                ("disk_logical_unit_number", int),
                ("disk_caching", str),
                ("disk_create_option", str),
                ("disk_delete_option", str),
                ("disk_id", str),
                ("storage_account_type", str),
            ],
        )
    ] = None,
    resource_id: str = None,
    subscription_id: str = None,
    tags: Dict[str, str] = None,
) -> Dict:
    r"""Create or update Virtual Machines.

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group.
        resource_id(str, Optional): Virtual Machine resource id on Azure
        virtual_machine_name(str): The name of the virtual machine.
        location(str): Resource location. Changing this forces a new resource to be created.
        virtual_machine_size(str): Specifies the size of the Virtual Machine.
        network_interface_ids(list[str]): A list of Network Interface IDs which should be associated with the Virtual Machine.
        subscription_id(str, Optional): Subscription Unique id.
        tags(dict[str, str], Optional): Resource tags.
        os_profile(dict[str, Any]): Specifies the operating system settings used while creating the virtual machine.

            * computer_name(str):
                Specifies the name of the Virtual Machine.
            * admin_username(str):
                Specifies the name of the local administrator account.
            * admin_password(str):
                (Required for Windows, Optional for Linux) The password associated with the local administrator account.
        storage_image_reference(dict[str, Any]): Specifies information about the image to use. Eg- platform images, marketplace images.

            * image_sku(str):
                Specifies the SKU of the image used to create the virtual machine. Changing this forces a new resource to be created.
            * image_publisher(str):
                Specifies the publisher of the image used to create the virtual machine. Changing this forces a new resource to be created.
            * image_version(str):
                Specifies the version of the image used to create the virtual machine. Changing this forces a new resource to be created.
            * image_offer(str, Optional):
                Specifies the offer of the image used to create the virtual machine. Changing this forces a new resource to be created.
        storage_os_disk(dict[str, Any]): Specifies information about the operating system disk used by the virtual machine.

            * disk_name(str):
                Specifies the name of the OS Disk.
            * disk_create_option(str):
                Specifies how the OS Disk should be created. Possible values are Attach (managed disks only) and FromImage.
            * storage_account_type(str, Optional):
                Specifies the type of Managed Disk which should be created. Possible values are Standard_LRS, StandardSSD_LRS or Premium_LRS.
            * disk_caching(str, Optional):
                Specifies the caching requirements for the OS Disk. Possible values include None, ReadOnly and ReadWrite.
            * disk_size_in_GB(str, Optional):
                Specifies the size of the OS Disk in gigabytes.
            * disk_delete_option(str, Optional):
                Specifies how the OS Disk should be handled after VM deletion. Possible values are Detach and Delete.
            * disk_id(str, Optional):
                Specifies the ID of an existing Managed Disk which should be attached as the OS Disk of this Virtual Machine. If this is set then the create_option must be set to Attach.
        storage_data_disks(list(dict[str, Any]), Optional): List of Data disks attached/added to a VM.

            * disk_name(str):
                The name of the Data Disk.
            * disk_create_option(str):
                Specifies how the data disk should be created. Possible values are Attach, FromImage and Empty.
            * disk_logical_unit_number(str):
                Specifies the logical unit number of the data disk. This needs to be unique within all the Data Disks on the Virtual Machine.
            * storage_account_type(str, Optional):
                Specifies the type of managed disk to create. Possible values are either Standard_LRS, StandardSSD_LRS, Premium_LRS or UltraSSD_LRS.
            * disk_caching(str, Optional):
                Specifies the caching requirements for the Data Disk. Possible values include None, ReadOnly and ReadWrite.
            * disk_size_in_GB(str, Optional):
                Specifies the size of the data disk in gigabytes.
            * disk_delete_option(str, Optional):
                Specifies how the OS Disk should be handled after VM deletion. Possible values are Detach and Delete.
            * disk_id(str, Optional):
                Specifies the ID of an Existing Managed Disk which should be attached to this Virtual Machine. When this field is set create_option must be set to Attach.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_present:
              azure.compute.virtual_machines.present:
                - name: my-vm
                - resource_group_name: my-rg-1
                - virtual_machine_name: my-vm
                - location: eastus
                - virtual_machine_size: Standard_B1ls
                - network_interface_ids:
                  - /subscriptions/subscription_id/resourceGroups/my-rg-1/providers/Microsoft.Network/networkInterfaces/my-nic-id-1
                - storage_image_reference:
                    image_sku: 18.04-LTS
                    image_publisher: Canonical
                    image_version: latest
                    image_offer: UbuntuServer
                - storage_os_disk:
                    storage_account_type: Standard_LRS
                    disk_name: my-os-disk
                    disk_caching: ReadWrite
                    disk_size_in_GB: 30
                    disk_create_option: FromImage
                    disk_delete_option: Detach
                - storage_data_disks:
                  - disk_name: my-data-disk
                    disk_size_in_GB: 2
                    disk_logical_unit_number: 0
                    disk_caching: None
                    disk_create_option: Empty
                    disk_delete_option: Delete
                - os_profile:
                    admin_username: my-admin-username
                    computer_name: machine-name
                    admin_password: Vmwareadmin123!
                - tags:
                    my-tag-key-1: my-tag-value-1
                    my-tag-key-2: my-tag-value-2
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }
    if subscription_id is None:
        subscription_id = ctx.acct.subscription_id
    if resource_id is None:
        resource_id = (
            f"/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}"
            f"/providers/Microsoft.Compute/virtualMachines/{virtual_machine_name}"
        )
    response_get = await hub.exec.azure.compute.virtual_machines.get(
        ctx, resource_id=resource_id, raw=True
    )
    if not response_get["result"]:
        hub.log.debug(
            f"Could not get azure.compute.virtual_machines {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result
    if not response_get["ret"]:
        if ctx.get("test", False):
            # Return a proposed state by Idem state --test
            result["new_state"] = hub.tool.azure.test_state_utils.generate_test_state(
                enforced_state={},
                desired_state={
                    "name": name,
                    "resource_group_name": resource_group_name,
                    "subscription_id": subscription_id,
                    "virtual_machine_name": virtual_machine_name,
                    "virtual_machine_size": virtual_machine_size,
                    "tags": tags,
                    "location": location,
                    "network_interface_ids": network_interface_ids,
                    "storage_image_reference": storage_image_reference,
                    "storage_os_disk": storage_os_disk,
                    "storage_data_disks": storage_data_disks,
                    "os_profile": os_profile,
                    "resource_id": resource_id,
                },
            )
            result["comment"].append(
                f"Would create azure.compute.virtual_machines '{name}'"
            )
            return result

        else:
            # PUT operation to create a resource
            payload = hub.tool.azure.compute.virtual_machines.convert_present_to_raw_virtual_machine(
                location=location,
                virtual_machine_size=virtual_machine_size,
                network_interface_ids=network_interface_ids,
                storage_image_reference=storage_image_reference,
                storage_os_disk=storage_os_disk,
                storage_data_disks=storage_data_disks,
                os_profile=os_profile,
                tags=tags,
            )

            response_put = await hub.exec.request.json.put(
                ctx,
                url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2022-03-01",
                success_codes=[200, 201],
                json=payload,
            )
            if not response_put["result"]:
                hub.log.debug(
                    f"Could not create azure.compute.virtual_machines {response_put['comment']} {response_put['ret']}"
                )
                result["comment"] = [response_put["comment"], response_put["ret"]]
                result["result"] = False
                return result

            result[
                "new_state"
            ] = hub.tool.azure.compute.virtual_machines.convert_raw_virtual_machine_to_present(
                resource=response_put["ret"],
                idem_resource_name=name,
                subscription_id=subscription_id,
                resource_group_name=resource_group_name,
                virtual_machine_name=virtual_machine_name,
                resource_id=resource_id,
            )
            result["comment"].append(f"Created azure.compute.virtual_machines '{name}'")
            return result
    else:
        existing_resource = response_get["ret"]
        result[
            "old_state"
        ] = hub.tool.azure.compute.virtual_machines.convert_raw_virtual_machine_to_present(
            resource=existing_resource,
            idem_resource_name=name,
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            virtual_machine_name=virtual_machine_name,
            resource_id=resource_id,
        )
        # Generate a new PUT operation payload with new values
        new_payload = (
            hub.tool.azure.compute.virtual_machines.update_virtual_machine_payload(
                response_get["ret"],
                {
                    "virtual_machine_size": virtual_machine_size,
                    "storage_data_disks": storage_data_disks,
                    "network_interface_ids": network_interface_ids,
                    "storage_os_disk": storage_os_disk,
                    "os_profile": os_profile,
                    "storage_image_reference": storage_image_reference,
                    "tags": tags,
                },
            )
        )
        if ctx.get("test", False):
            if new_payload["ret"] is None:
                result["new_state"] = copy.deepcopy(result["old_state"])
                result["comment"].append(
                    f"azure.compute.virtual_machines '{name}' has no property need to be updated."
                )
            else:
                result[
                    "new_state"
                ] = hub.tool.azure.compute.virtual_machines.convert_raw_virtual_machine_to_present(
                    resource=new_payload["ret"],
                    idem_resource_name=name,
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    virtual_machine_name=virtual_machine_name,
                    resource_id=resource_id,
                )
                result["comment"].append(
                    f"Would update azure.compute.virtual_machines '{name}'"
                )
            return result
        # PUT operation to update a resource
        if new_payload["ret"] is None:
            result["new_state"] = copy.deepcopy(result["old_state"])
            result["comment"].append(
                f"azure.compute.virtual_machines '{name}' has no property need to be updated."
            )
            return result
        result["comment"] += new_payload["comment"]

        response_put = await hub.exec.request.json.put(
            ctx,
            url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2022-03-01",
            success_codes=[200, 201],
            json=new_payload["ret"],
        )
        if not response_put["result"]:
            hub.log.debug(
                f"Could not update azure.compute.virtual_machines {response_put['comment']} {response_put['ret']}"
            )
            result["result"] = False
            result["comment"] = [response_put["comment"], response_put["ret"]]
            return result

        result[
            "new_state"
        ] = hub.tool.azure.compute.virtual_machines.convert_raw_virtual_machine_to_present(
            resource=response_put["ret"],
            idem_resource_name=name,
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            virtual_machine_name=virtual_machine_name,
            resource_id=resource_id,
        )
        if result["old_state"] == result["new_state"]:
            result["comment"].append(
                f"azure.compute.virtual_machines '{name}' has no property need to be updated."
            )
            return result

        result["comment"].append(f"Updated azure.compute.virtual_machines '{name}'")
        return result


async def absent(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    virtual_machine_name: str,
    subscription_id: str = None,
) -> Dict:
    r"""Delete a Virtual Machine.

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group.
        virtual_machine_name(str): The name of the virtual machine.
        subscription_id(str, Optional): Subscription Unique id.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_absent:
              azure.compute.virtual_machines.absent:
                - name: my-vm
                - resource_group_name: my-resource-group
                - virtual_machine_name: my-vm
                - subscription_id: my-subscription
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }
    if subscription_id is None:
        subscription_id = ctx.acct.subscription_id
    resource_id = (
        f"/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}"
        f"/providers/Microsoft.Compute/virtualMachines/{virtual_machine_name}"
    )
    response_get = await hub.exec.azure.compute.virtual_machines.get(
        ctx,
        resource_id=resource_id,
    )
    if not response_get["result"]:
        hub.log.debug(
            f"Could not get azure.compute.virtual_machines '{name}' {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result

    if response_get["ret"]:
        result["old_state"] = response_get["ret"]
        result["old_state"]["name"] = name

        if ctx.get("test", False):
            result["comment"].append(
                f"Would delete azure.compute.virtual_machines '{name}'"
            )
            return result

        response_delete = await hub.exec.request.raw.delete(
            ctx,
            url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2022-03-01",
            success_codes=[200, 202, 204],
        )

        if not response_delete["result"]:
            hub.log.debug(
                f"Could not delete azure.compute.virtual_machines {response_delete['comment']} {response_delete['ret']}"
            )
            result["result"] = False
            result["comment"] = [response_delete["comment"], response_delete["ret"]]
            return result

        result["comment"].append(f"Deleted azure.compute.virtual_machines '{name}'")
        return result

    else:
        # If Azure returns 'Not Found' error, it means the resource has been absent.
        result["comment"].append(
            f"azure.compute.virtual_machines '{name}' already absent"
        )
        return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""Describe the resource in a way that can be recreated/managed with the corresponding "present" function.

    Lists all Virtual Machines under the same subscription.


    Returns:
        Dict[str, Any]

    Examples:
        .. code-block:: bash

            $ idem describe azure.compute.virtual_machines
    """
    result = {}
    ret_list = await hub.exec.azure.compute.virtual_machines.list(ctx)
    if not ret_list["ret"]:
        hub.log.debug(
            f"Could not describe compute virtual_machines {ret_list['comment']}"
        )
        return result

    for resource in ret_list["ret"]:
        resource_id = resource["resource_id"]
        result[resource_id] = {
            "azure.compute.virtual_machines.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in resource.items()
            ]
        }
    return result



"""States module for managing Firewall Policy."""
import copy
from dataclasses import field
from dataclasses import make_dataclass
from typing import Any
from typing import Dict
from typing import List

__contracts__ = ["resource"]
__reconcile_wait__ = {"static": {"wait_in_seconds": 20}}


async def present(
    hub,
    ctx,
    name: str,
    location: str,
    resource_group_name: str,
    firewall_policy_name: str,
    subscription_id: str = None,
    tags: Dict = None,
    base_policy_id: str = None,
    threat_intelligence_allow_list: make_dataclass(
        "threatIntelligenceAllowList",
        [
            ("fqdns", List[str], field(default=None)),
            ("ip_addresses", List[str], field(default=None)),
        ],
    ) = None,
    dns_settings: make_dataclass(
        "dnsSettings",
        [
            ("proxy_enabled", bool, field(default=None)),
            ("servers", List[str], field(default=None)),
        ],
    ) = None,
    sku: make_dataclass(
        "sku",
        [("tier", str, field(default=None))],
    ) = None,
    intrusion_detection: make_dataclass(
        "intrusionDetection",
        [("mode", str)],
    ) = None,
    threat_intelligence_mode: str = None,
    resource_id: str = None,
) -> Dict:
    r"""Create or update Firewall Policy.

    Args:
        name(str): The identifier for this state.
        location(str): Resource location. Changing this forces a new resource to be created.
        resource_group_name(str): The name of the resource group.
        subscription_id(str, Optional): Subscription Unique id.
        resource_id(str, Optional): Firewall Policy resource id on Azure
        tags(dict[str, str], Optional): Resource tags.
        firewall_policy_name(str): The name of the firewall policy.
        base_policy_id(str, Optional): The ID of the base Firewall Policy.
        threat_intelligence_allow_list(dict[str, Any], Optional): Specifies threat_intelligence_allowlist while creating the Firewall policy.

            * fqdns(list[str]):
                A list of FQDNs that will be skipped for threat detection.
            * ip_addresses(list[str]):
                A list of IP addresses or CIDR ranges that will be skipped for threat detection.
        dns_settings(dict[str, Any], Optional): Specifies dns setting while creating the Firewall policy.

            * proxy_enabled(bool):
                Whether to enable DNS proxy on Firewalls attached to this Firewall Policy.
            * servers(list[str]):
                A list of custom DNS servers' IP addresses.
        sku(dict[str, Any], Optional): The SKU Tier of the Firewall Policy.

            * tier(str):
                Possible values are Standard, Premium and Basic. Changing this forces a new Firewall Policy to be created.
        intrusion_detection(dict[str, Any], Optional): A intrusion_detection block for Firewall policy.

            * mode(str):
                In which mode you want to run intrusion detection: Off, Alert or Deny.
        threat_intelligence_mode(str, Optional): The operation mode for Threat Intelligence. Possible values are Alert, Deny and Off. Defaults to Alert.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_present:
              azure.network.firewall_policies.present:
                - name: my_fp
                - subscription_id: my_sub_id
                - resource_group_name: my_rg-1
                - firewall_policy_name: my-fp
                - location: eastus
                - tags:
                    key: value
                - sku:
                    tier: Premium
                - base_policy_id: my_base_pol_id
                - threat_intelligence_allow_list:
                    ip_addresses:
                      - my_ip_address
                    fqdns:
                      - "*"
                - intrusion_detection:
                    mode: Alert
                - threat_intelligence_mode: Alert
                - dns_settings:
                    proxy_enabled: true
                    servers:
                      - my_server
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }
    if subscription_id is None:
        subscription_id = ctx.acct.subscription_id
    if resource_id is None:
        resource_id = f"/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/firewallPolicies/{firewall_policy_name}"

    response_get = await hub.exec.azure.network.firewall_policies.get(
        ctx, resource_id=resource_id, raw=True
    )
    if not response_get["result"]:
        hub.log.debug(
            f"Could not get azure.network.firewall_policies {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result

    if not response_get["ret"]:
        if ctx.get("test", False):
            # Return a proposed state by Idem state --test
            result["new_state"] = hub.tool.azure.test_state_utils.generate_test_state(
                enforced_state={},
                desired_state={
                    "name": name,
                    "resource_group_name": resource_group_name,
                    "subscription_id": subscription_id,
                    "firewall_policy_name": firewall_policy_name,
                    "tags": tags,
                    "location": location,
                    "resource_id": resource_id,
                    "dns_settings": dns_settings,
                    "sku": sku,
                    "intrusion_detection": intrusion_detection,
                    "threat_intelligence_mode": threat_intelligence_mode,
                    "threat_intelligence_allow_list": threat_intelligence_allow_list,
                    "base_policy_id": base_policy_id,
                },
            )
            result["comment"].append(
                f"Would create azure.network.firewall_policies '{name}'"
            )
            return result
        else:
            # PUT operation to create a resource
            payload = hub.tool.azure.network.firewall_policies.convert_present_to_raw_fire_policies(
                location=location,
                tags=tags,
                dns_settings=dns_settings,
                sku=sku,
                intrusion_detection=intrusion_detection,
                threat_intelligence_mode=threat_intelligence_mode,
                threat_intelligence_allow_list=threat_intelligence_allow_list,
                base_policy_id=base_policy_id,
            )
            response_put = await hub.exec.request.json.put(
                ctx,
                url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2022-07-01",
                success_codes=[200, 201],
                json=payload,
            )
            if not response_put["result"]:
                hub.log.debug(
                    f"Could not create firewall policy {response_put['comment']} {response_put['ret']}"
                )
                result["comment"] = [response_put["comment"], response_put["ret"]]
                result["result"] = False
                return result

            result[
                "new_state"
            ] = hub.tool.azure.network.firewall_policies.convert_raw_firewall_policies_to_present(
                resource=response_put["ret"],
                idem_resource_name=name,
                resource_group_name=resource_group_name,
                firewall_policy_name=firewall_policy_name,
                resource_id=resource_id,
                subscription_id=subscription_id,
            )
            result["comment"].append(
                f"Created azure.network.firewall_policies '{name}'"
            )
            return result

    else:
        existing_resource = response_get["ret"]
        result[
            "old_state"
        ] = hub.tool.azure.network.firewall_policies.convert_raw_firewall_policies_to_present(
            resource=existing_resource,
            idem_resource_name=name,
            resource_group_name=resource_group_name,
            firewall_policy_name=firewall_policy_name,
            resource_id=resource_id,
            subscription_id=subscription_id,
        )
        # Generate a new PUT operation payload with new values
        new_payload = (
            hub.tool.azure.network.firewall_policies.update_fire_policies_payload(
                existing_resource, {"tags": tags}
            )
        )
        if ctx.get("test", False):
            if new_payload["ret"] is None:
                result["new_state"] = copy.deepcopy(result["old_state"])
                result["comment"].append(
                    f"azure.network.firewall_policies '{name}' doesn't need to be updated."
                )
            else:
                result[
                    "new_state"
                ] = hub.tool.azure.network.firewall_policies.convert_raw_firewall_policies_to_present(
                    resource=new_payload["ret"],
                    idem_resource_name=name,
                    resource_group_name=resource_group_name,
                    firewall_policy_name=firewall_policy_name,
                    resource_id=resource_id,
                    subscription_id=subscription_id,
                )
                result["comment"].append(
                    f"Would update azure.network.firewall_policies '{name}'"
                )
            return result

        # PUT operation to update a resource
        if new_payload["ret"] is None:
            result["new_state"] = copy.deepcopy(result["old_state"])
            result["comment"].append(
                f"azure.network.firewall_policies '{name}' doesn't need to be updated."
            )
            return result
        result["comment"] = result["comment"] + new_payload["comment"]
        response_put = await hub.exec.request.json.put(
            ctx,
            url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2021-06-01",
            success_codes=[200, 201],
            json=new_payload["ret"],
        )
        if not response_put["result"]:
            hub.log.debug(
                f"Could not update azure.network.firewall_policies {response_put['comment']} {response_put['ret']}"
            )
            result["result"] = False
            result["comment"] = [response_put["comment"], response_put["ret"]]
            return result

        result[
            "new_state"
        ] = hub.tool.azure.network.firewall_policies.convert_raw_firewall_policies_to_present(
            resource=response_put["ret"],
            idem_resource_name=name,
            resource_group_name=resource_group_name,
            firewall_policy_name=firewall_policy_name,
            resource_id=resource_id,
            subscription_id=subscription_id,
        )
        result["comment"].append(f"Updated azure.network.firewall_policies '{name}'")
        return result


async def absent(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    firewall_policy_name: str,
    subscription_id: str = None,
) -> dict:
    r"""Delete a firewall policy.

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group.
        firewall_policy_name(str): The name of the firewall policy.
        subscription_id(str, Optional): Subscription Unique id.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_absent:
              azure.network.firewall_policies.absent:
                - name: my-fp
                - subscription_id: my-subscription
                - resource_group_name: my-resource-group
                - firewall_policy_name: my-fp
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }
    if subscription_id is None:
        subscription_id = ctx.acct.subscription_id
    resource_id = f"/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/firewallPolicies/{firewall_policy_name}"
    response_get = await hub.exec.azure.network.firewall_policies.get(
        ctx,
        resource_id=resource_id,
    )
    if not response_get["result"]:
        hub.log.debug(
            f"Could not get azure.network.firewall_policies '{name}' {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result

    if response_get["ret"]:
        result["old_state"] = response_get["ret"]
        result["old_state"]["name"] = name
        if ctx.get("test", False):
            result["comment"].append(
                f"Would delete azure.network.firewall_policies '{firewall_policy_name}'"
            )
            return result
        response_delete = await hub.exec.request.raw.delete(
            ctx,
            url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2022-07-01",
            success_codes=[200, 202],
        )

        if not response_delete["result"]:
            hub.log.debug(
                f"Could not delete azure.network.firewall_policies {response_delete['comment']} {response_delete['ret']}"
            )
            result["result"] = False
            result["comment"] = [response_delete["comment"], response_delete["ret"]]
            return result

        result["comment"].append(
            f"Deleted azure.network.firewall_policies '{firewall_policy_name}'"
        )
        return result
    else:
        # If Azure returns 'Not Found' error, it means the resource has been absent.
        result["comment"].append(
            f"azure.network.firewall_policies '{firewall_policy_name}' already absent"
        )
        return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""Describe the resource in a way that can be recreated/managed with the corresponding "present" function.

    Lists all firewall policy under the same subscription.


    Returns:
        Dict[str, Any]

    Examples:
        .. code-block:: bash

            $ idem describe azure.network.firewall_policies
    """
    result = {}
    ret = await hub.exec.azure.network.firewall_policies.list(ctx)
    if not ret["result"]:
        hub.log.debug(f"Could not describe network firewall_policies {ret['comment']}")
        return {}

    for resource in ret["ret"]:
        resource_id = resource.get("resource_id")
        result[resource_id] = {
            f"azure.network.firewall_policies.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in resource.items()
            ]
        }
    return result



"""State module for managing Authorization Role Assignment."""
import copy
import uuid
from typing import Any
from typing import Dict


__contracts__ = ["resource"]


async def present(
    hub,
    ctx,
    name: str,
    scope: str,
    role_definition_id: str,
    principal_id: str,
    resource_id: str = None,
    role_assignment_name: str = None,
) -> Dict[str, Any]:
    r"""Create or update Role Assignments.

    Args:
        name(str): The identifier for this state.
        scope(str): The scope of the role assignment to create. The scope can be any REST resource instance.
            For example, use '/subscriptions/{subscription-id}/' for a subscription,
            '/subscriptions/{subscription-id}/resourceGroups/{resource-group-name}' for a resource group,
            and '/subscriptions/{subscription-id}/resourceGroups/{resource-group-name}/providers/{resource-provider}/{resource-type}/{resource-name}' for a resource.
        role_definition_id(str): The role definition ID used in the role assignment.
        principal_id(str): The principal ID assigned to the role. This maps to the ID inside the Active Directory. It can point to a user, service principal, or security group.
        resource_id(str, Optional): Role Assignment resource id on Azure.
        role_assignment_name(str, Optional): A GUID for the role assignment to create. The name must be unique and different for each role assignment. This will be automatically generated if not specified.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_present:
              azure.authorization.role_assignments.present:
                - name: value
                - scope: value
                - role_assignment_name: value
    """
    result = {
        "name": name,
        "result": True,
        "old_state": None,
        "new_state": None,
        "comment": [],
    }
    response_get = None

    if role_assignment_name:
        if resource_id is None:
            resource_id = f"{scope}/providers/Microsoft.Authorization/roleAssignments/{role_assignment_name}"

        response_get = await hub.exec.azure.authorization.role_assignments.get(
            ctx, resource_id=resource_id, raw=True
        )

    if response_get["result"]:
        if not response_get["ret"]:
            if role_assignment_name is None:
                role_assignment_name = uuid.uuid4()
            if ctx.get("test", False):
                # Return a proposed state by Idem state --test
                result[
                    "new_state"
                ] = hub.tool.azure.test_state_utils.generate_test_state(
                    enforced_state={},
                    desired_state={
                        "name": name,
                        "scope": scope,
                        "role_assignment_name": role_assignment_name,
                        "resource_id": resource_id,
                        "role_definition_id": role_definition_id,
                        "principal_id": principal_id,
                    },
                )
                result["comment"].append(
                    f"Would create azure.authorization.role_assignments '{name}'"
                )
                return result
            else:
                # PUT operation to create a resource
                payload = hub.tool.azure.authorization.role_assignments.convert_present_to_raw_role_assignments(
                    role_definition_id=role_definition_id,
                    principal_id=principal_id,
                )
                response_put = await hub.exec.request.json.put(
                    ctx,
                    url=f"{ctx.acct.endpoint_url}{resource_id}?api-version=2015-07-01",
                    success_codes=[201],
                    json=payload,
                )

                if not response_put["result"]:
                    hub.log.debug(
                        f"Could not create azure.authorization.role_assignments {response_put['comment']} {response_put['ret']}"
                    )
                    result["comment"] = [response_put["comment"], response_put["ret"]]
                    result["result"] = False
                    return result

                result[
                    "new_state"
                ] = hub.tool.azure.authorization.role_assignments.convert_raw_role_assignments_to_present(
                    resource=response_put["ret"],
                    idem_resource_name=name,
                    role_assignment_name=role_assignment_name,
                    resource_id=resource_id,
                )
                result["comment"].append(
                    f"Created azure.authorization.role_assignments '{name}'"
                )
                return result
        else:
            existing_resource = response_get["ret"]
            result[
                "old_state"
            ] = hub.tool.azure.authorization.role_assignments.convert_raw_role_assignments_to_present(
                resource=existing_resource,
                idem_resource_name=name,
                role_assignment_name=role_assignment_name,
                resource_id=resource_id,
            )
            # No role assignment property can be updated without resource re-creation.
            result["comment"].append(
                f"azure.authorization.role_assignments '{name}' has no property to be updated."
            )
            result["new_state"] = copy.deepcopy(result["old_state"])
            return result
    else:
        hub.log.debug(
            f"Could not get azure.authorization.role_assignments {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
        return result


async def absent(
    hub, ctx, name: str, scope: str, role_assignment_name: str, resource_id: str = None
) -> Dict[str, Any]:
    r"""Delete Role Assignments.

    Args:
        name(str): The identifier for this state.
        scope(str, Optional): The scope of the role assignment to delete.
        role_assignment_name(str, Optional): The name of the role assignment to delete.
        resource_id(str, Optional): Role assignment resource id on Azure. Either resource_id or a combination of scope
            and role_assignment_name need to be specified. Idem will automatically consider a resource as absent if both
            options are not specified.

    Returns:
        Dict

    Examples:
        .. code-block:: sls

            resource_is_absent:
              azure.authorization.role_assignments.absent:
                - name: value
                - scope: value
                - role_assignment_name: value
    """
    result = dict(name=name, result=True, comment=[], old_state=None, new_state=None)
    if scope is not None and role_assignment_name is not None:
        constructed_resource_id = f"{scope}/providers/Microsoft.Authorization/roleAssignments/{role_assignment_name}"
        if resource_id is not None and resource_id != constructed_resource_id:
            result["result"] = False
            result["comment"].append(
                f"azure.authorization.role_assignments '{name}' resource_id {resource_id} does not match the constructed resource id"
            )
            return result
        resource_id = constructed_resource_id

    response_get = await hub.exec.azure.authorization.role_assignments.get(
        ctx,
        resource_id=resource_id,
    )
    if response_get["result"]:
        if response_get["ret"]:
            result["old_state"] = response_get["ret"]
            result["old_state"]["name"] = name
            if ctx.get("test", False):
                result["comment"].append(
                    f"Would delete azure.authorization.role_assignments '{name}'"
                )
                return result
            response_delete = await hub.exec.request.raw.delete(
                ctx,
                url=f"{ctx.acct.endpoint_url}/{resource_id}?api-version=2015-07-01",
                success_codes=[200, 204],
            )

            if not response_delete["result"]:
                hub.log.debug(
                    f"Could not delete azure.authorization.role_assignments '{name}' {response_delete['comment']} {response_delete['ret']}"
                )
                result["result"] = False
                result["comment"] = [response_delete["comment"], response_delete["ret"]]
                return result

            result["comment"].append(
                f"Deleted azure.authorization.role_assignments '{name}'"
            )
            return result

        else:
            # If Azure returns 'Not Found' error, it means the resource has been absent.
            result["comment"].append(
                f"azure.authorization.role_assignments '{name}' already absent"
            )
    else:
        hub.log.debug(
            f"Could not get azure.authorization.role_assignments '{name}' {response_get['comment']} {response_get['ret']}"
        )
        result["result"] = False
        result["comment"] = [response_get["comment"], response_get["ret"]]
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""Describe the resource in a way that can be recreated/managed with the corresponding "present" function.

    Lists all Role Assignments under the same subscription.

    Returns:
        Dict[str, Any]

    Examples:
        .. code-block:: bash

            $ idem describe azure.authorization.role_assignments
    """
    result = {}
    ret_list = await hub.exec.azure.authorization.role_assignments.list(ctx)
    if not ret_list["ret"]:
        hub.log.debug(f"Could not describe role assignment {ret_list['comment']}")
        return result

    for resource in ret_list["ret"]:
        resource_id = resource["resource_id"]
        result[resource_id] = {
            "azure.authorization.role_assignments.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in resource.items()
            ]
        }

    return result
